/********************************************************************************
*                                                                               *
* Copyright (c) 2007, Computer Networks and Internet, University of Tübingen,   *
* Sand 13, 72076 Tübingen, Germany                                              *
*                                                                               *
* All rights reserved.                                                          *
*                                                                               *
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Lesser General Public                    *
* License as published by the Free Software Foundation; either                  *
* version 2.1 of the License, or (at your option) any later version.            *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU              *
* Lesser General Public License for more details.                               *
*                                                                               *
* You should have received a copy of the GNU Lesser General Public              *
* License along with this library; if not, write to the Free Software           *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA       *
*                                                                               *
********************************************************************************/
#ifndef PAM_CONFIGURATION_H_
#define PAM_CONFIGURATION_H_

namespace pam_xacml {

//How to obtain the XACML request
enum RequestBuilderType {
	INTERNAL_REQUEST_BUILDER,
	APPLICATION_REQUEST_BUILDER,
	SCRIPT_REQUEST_BUILDER,
	SOAP_REQUEST_BUILDER
};

//How to request decision
enum PdpRequesterType {
	FILE_PDP_REQUESTER,
    SIMPLE_PDP_REQUESTER,
    SOAP_PDP_REQUESTER,
    HANNES_PDP_REQUESTER,
    BESTER_PDP_REQUESTER
};

class PAMConfiguration { // : public Singleton<PAMConfiguration> {
	//friend class Singleton<PAMConfiguration>;
/*
private:
	PAMConfiguration _singleton;

protected:
*/
public:
	PAMConfiguration();
	virtual ~PAMConfiguration();

public:
	void parsePamArguments(int argc, const char** argv);

	const char* getApplicationName();

	RequestBuilderType getRequestBuilder();
	PdpRequesterType getPdpRequester();
	const char* getStaticResponse();
	const char* getPdpEndpoint();

	const char* getRequestResource();
	const char* getUsername();
	const char* getRequestAction();

	bool isMandatoryObligations();
	bool isObligationsIgnore();

	bool isDebug();
	bool isUseSSL();

	const char* getFileCAchain();
	const char* getFileClientCertKey();
	const char* getFileClientPassword();
	const char* getClientPassword();
	const char* getRequestTemplate();
	const char* getObligationsScript();
	const char* getRequestBuilderScript();
	const char* getPDPFormat();
	const char* getLdapConf();

	void setUsername(char* username);
	void setAppname(char* appname);

private:
	//builder and requester
	RequestBuilderType requestBuilder;
	PdpRequesterType pdpRequester;

	//parameters
	char* applicationName;
	char* pdpEndpoint;//SIMPLE_PDP_REQUESTER IP address:port and for SOAP_PDP_REQUESTER the endpoint URL
	char* requestResource;//optional for INTERNAL_REQUEST_BUILDER
	char* requestAction;//optional for INTERNAL_REQUEST_BUILDER
	char* username;//optional for SOAP_REQUEST_BUILDER
	char* staticResponse;//mandatory for FILE_PDP_REQUESTER
	bool mandatoryObligations;//optional for APPLICATION_REQUEST_BUILDER
	bool obligationsIgnore;
	char* obligationsScript;
	char* requestBuilderScript;
	char* ldapConf;
	char* ou;
	char* pdpFormat;

	//debgugging
	bool debug;
	bool dumpXacml;

	//security
	bool use_ssl;
	char* file_ca_chain;
	char* file_client_cert_key;
	char* file_client_password;
	char* client_password;


	// templates
	char* requestTemplate;


protected:
	void validateConfiguration();
	void readSSLPassphrase(char* file_client_password, int debug, char** client_password);
	char* getValueOfOption(char** savePtr, const char* tok);

};



}

#endif
