/********************************************************************************
*                                                                               *
* Copyright (c) 2011, ThereSIS, Romain FERRARI,   								*
* Palaiseau, France                                              				*
*                                                                               *
* All rights reserved.                                                          *
*                                                                               *
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Lesser General Public                    *
* License as published by the Free Software Foundation; either                  *
* version 2.1 of the License, or (at your option) any later version.            *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU              *
* Lesser General Public License for more details.                               *
*                                                                               *
* You should have received a copy of the GNU Lesser General Public              *
* License along with this library; if not, write to the Free Software           *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA       *
*                                                                               *
********************************************************************************/
/**
*
*   define
*
**/
//#define DEBUG   1


/**
*
*   Includes
*
**/
//#include "soapPDPImplPortBindingProxy.h"
//#include "PDPImplPortBinding.nsmap"
//#include "soapClient.cpp"
//#include "xacml_datatypes.h"
//#include "xacml.h"
//#include "xacml_i.h"

#include <iostream>

//#include "../restClient/source/restclient.cpp"

//#ifdef USE_CURL
#include <curl/curl.h>
//#endif

// Standard C includes
#include <stdlib.h>

#include <syslog.h>

//C++ includes
#include <stdexcept>
#include <string>
#include <unistd.h>

// DOM Includes
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/dom/DOM.hpp>
#include <xercesc/framework/StdOutFormatTarget.hpp>
#include <xercesc/framework/MemBufFormatTarget.hpp>
#include <xercesc/framework/MemBufInputSource.hpp>
#include <xercesc/parsers/XercesDOMParser.hpp>
#include <xercesc/dom/DOMAttr.hpp>

#include "pdpSOAPRequester.h"

using namespace xercesc_3_1;

struct MemoryStruct 
{
	char *memory;
	size_t size;
};

namespace pam_xacml
{

//static size_t WriteMemoryCallback(void *contents, size_t size, size_t nmemb, void *userp)
//{
//	size_t realsize = size * nmemb;
//	struct MemoryStruct *mem = (struct MemoryStruct *)userp;
//	
//	mem->memory = (char*)realloc(mem->memory, mem->size + realsize + 1);
//	if (mem->memory == NULL) 
//	{
//		/* out of memory! */ 
//		printf("not enough memory (realloc returned NULL)\n");
//		exit(EXIT_FAILURE);
//	}
//	
//	memcpy(&(mem->memory[mem->size]), contents, realsize);
//	mem->size += realsize;
//	mem->memory[mem->size] = 0;
//	
//	return realsize;
//}

pdpSOAPRequester::pdpSOAPRequester(PAMConfiguration* conf) :  PDPRequester(conf)
{
	if (_conf->isDebug())
		syslog(LOG_INFO, " - Constructor");
	char* pdpEndpoint = strdup(_conf->getPdpEndpoint());
	char* bEndPoint = NULL;
	char* eEndPoint = NULL;
	char* host = NULL;
	char* s_port = NULL;
	int size;
	int PORT;
	if (_conf->isDebug())
		syslog(LOG_INFO, " - Variables initialized");

	size = strlen(pdpEndpoint);
	if (strstr(pdpEndpoint, "http"))
	{
		/**
		 * HTTP Endpoint
		 */

		bEndPoint = (char*)calloc(1, size);
		if (bEndPoint == NULL)
			exit(0);
		strncpy(bEndPoint, pdpEndpoint, size);
		bEndPoint += strlen("http://");
	}
	else
	{
		/**
		 * HTTPS Endpoint
		 */
		bEndPoint = (char*)calloc(1, strlen(pdpEndpoint+1));
		if (bEndPoint == NULL)
			exit(0);
		strncpy(bEndPoint, pdpEndpoint, size);
		bEndPoint += strlen("https://");
	}
	eEndPoint = strstr(bEndPoint, ":");
	size = strlen(bEndPoint) - strlen(eEndPoint);
	host = (char*)calloc(1, size+1);
	if (host == NULL)
		exit(0);
	strncpy(host, bEndPoint, size);
	host[size]='\0';
	if (_conf->isDebug())
		syslog(LOG_INFO, " - host: %s", host);
	bEndPoint = strchr(bEndPoint, ':');
	bEndPoint += 1;
	eEndPoint = strchr(bEndPoint, '/');
	size = strlen(bEndPoint) - strlen(eEndPoint);
	s_port = (char*)calloc(1, size);
	if (s_port == NULL)
		exit(0);
	strncpy(s_port, bEndPoint, size);
	s_port[size] = '\0';

	if (_conf->isDebug())
		syslog(LOG_INFO, " - " __FILE__ " - using %s as endpoint and %s as port", pdpEndpoint, s_port);

	if (s_port == NULL) {
		throw std::runtime_error(__FILE__ " - could not find delimitier ':' in pdpEndpoint configuration directive (should look like http://localhost:1234/axis2/services/AuthzService)");
	}
	if (strlen(pdpEndpoint) == (s_port - pdpEndpoint+1)) {
		throw std::runtime_error(__FILE__ " - not port specified in pdpEndpoint configuration directive (should look like http://localhost:1234/axis2/services/AuthzService)");
	}

	PORT = atoi(s_port);

	this->sockfd = socket(AF_INET, SOCK_STREAM, 0);
	if (this->sockfd < 0)
		throw std::runtime_error("Could not open socket.");

	this->server = gethostbyname(host);
	if (this->server == NULL)
		throw std::runtime_error("Could not resolve PDP hostname.");

	bzero((char *) &(this->serv_addr), sizeof(this->serv_addr));
	this->serv_addr.sin_family = AF_INET;
	bcopy((char *)(this->server->h_addr), (char *)&(this->serv_addr.sin_addr.s_addr), this->server->h_length);
	serv_addr.sin_port = htons(PORT);

	if (connect(this->sockfd,(const sockaddr*)&(this->serv_addr),sizeof(this->serv_addr)) < 0)
		throw std::runtime_error("Could not connect to PDP.");

}

pdpSOAPRequester::~pdpSOAPRequester()
{
	close(this->sockfd);
}

std::string pdpSOAPRequester::outputXmlStream(DOMDocument* myDoc)
{
        DOMImplementation	*pImplement	= NULL;
//	DOMWriter		*pSerializer	= NULL;
        DOMLSSerializer         *pSerializer	= NULL;
	XMLFormatTarget		*pTarget	= NULL;
        DOMLSOutput             *pDomOutput     = NULL;        
        XMLCh*                  XMLString;
        std::string             xmlOutput       = "";                        
        
	/*
	Return the first registered implementation that has the desired features. In this case, we are after
	a DOM implementation that has the LS feature... or Load/Save.
	*/
        pImplement = DOMImplementationRegistry::getDOMImplementation(XMLString::transcode("LS"));        

	/*
	From the DOMImplementation, create a DOMWriter.
	DOMWriters are used to serialize a DOM tree [back] into an XML document.
	*/
        pSerializer = ((DOMImplementationLS*)pImplement)->createLSSerializer();
        

	/*
	This line is optional. It just sets a feature of the Serializer to make the output
	more human-readable by inserting line-feeds and tabs, without actually inserting any new nodes
	into the DOM tree. (There are many different features to set.) Comment it out and see the difference.
	*/
//	pSerializer->setFeature(XMLUni::fgDOMWRTFormatPrettyPrint, true);          

	/*
	Choose a location for the serialized output. The 3 options are:
		1) StdOutFormatTarget		(std output stream -  good for debugging)
		2) MemBufFormatTarget		(to Memory)
		3) LocalFileFormatTarget	(save to file)
		(Note: You'll need a different header file for each one)
	*/
        /* Ecriture dans la mémoire */        
        /* Ecriture sur la sortie standard */
        XMLString = pSerializer->writeToString(myDoc);        
                
        /* Libération des resources */
        pSerializer->release();
        //delete pTarget;
        
        
        return XMLString::transcode(XMLString);
        
}

/**
 * 
 * @param subject
 * @param action
 * @param resource
 * @return 
 */
std::string pdpSOAPRequester::buildXmlRequest(std::string _subject, std::string _resource, std::string _action)
{
    std::string xmlResult = "";
    std::string xmlns = "urn:oasis:names:tc:xacml:2.0:context:schema:os";
    std::string ns2 = "urn:oasis:names:tc:xacml:2.0:policy:schema:os";
    std::string datatype = "http://www.w3.org/2001/XMLSchema#string";
    std::string subjectAttribute = "urn:oasis:names:tc:xacml:1.0:subject:subject-id";
    std::string resourceAttribute = "urn:oasis:names:tc:xacml:1.0:resource:resource-id";
    std::string actionAttribute = "urn:oasis:names:tc:xacml:1.0:action:action-id";
    
    syslog(LOG_INFO, " - Building Request");
    try {
        XMLPlatformUtils::Initialize();
        
    } catch (const XMLException& e) {
        syslog(LOG_ERR, " - Error during DOM initialization");
        exit(1);
    }
    
    syslog(LOG_INFO, " - DOM Initialization");
    DOMImplementation * pDomImplementation = NULL;
    pDomImplementation = DOMImplementationRegistry::getDOMImplementation(XMLString::transcode("core"));


    DOMDocument*        myDoc = NULL;
    DOMElement*         root = NULL;
    DOMElement*         subject = NULL;    
    DOMElement*         resource = NULL;
    DOMElement*         action = NULL;
    DOMElement*         environment = NULL;
    DOMAttr*            namespaceNs2 = NULL;
        
    syslog(LOG_INFO, " - DOM Initialization OK, let's build");
  
    try {        
        myDoc = pDomImplementation->createDocument(XMLString::transcode(xmlns.c_str()), XMLString::transcode("Request"), 0);
//        namespaceNs2 = myDoc->createAttributeNS(XMLString::transcode(ns2.c_str()), XMLString::transcode("ns2"));        
//        namespaceNs2->setPrefix(XMLString::transcode("xmlns"));        
        
        
        root = myDoc->getDocumentElement();
//        root->setAttributeNS(XMLString::transcode(ns2.c_str()), XMLString::transcode("ns2"), XMLString::transcode(ns2.c_str()));
        
        /*Subject*/
        DOMText*        subjectText = NULL;
        DOMElement*     subjectAttr = NULL;
        DOMElement*     subjectAttrValue = NULL;
        DOMAttr*        subjectAttributeId = NULL;
        DOMAttr*        subjectDatatype = NULL;
                
        subject = myDoc->createElementNS(XMLString::transcode(xmlns.c_str()), XMLString::transcode("Subject"));
        subjectAttr = myDoc->createElementNS(XMLString::transcode(xmlns.c_str()), XMLString::transcode("Attribute"));
        subjectAttrValue = myDoc->createElementNS(XMLString::transcode(xmlns.c_str()), XMLString::transcode("AttributeValue"));
        subjectAttributeId = myDoc->createAttribute(XMLString::transcode("AttributeId"));
        subjectDatatype = myDoc->createAttribute(XMLString::transcode("DataType")); 
        subjectDatatype->setValue(XMLString::transcode(datatype.c_str()));
        subjectAttributeId->setValue(XMLString::transcode(subjectAttribute.c_str()));
        subjectText = myDoc->createTextNode(XMLString::transcode(_subject.c_str()));                        
        subjectAttr->setAttributeNode(subjectAttributeId);
        subjectAttr->setAttributeNode(subjectDatatype);
        subjectAttrValue->appendChild(subjectText);
        subjectAttr->appendChild(subjectAttrValue);
        subject->appendChild(subjectAttr);        
        
        /*resource*/
        DOMText*        resourceText = NULL;
        DOMElement*     resourceAttr = NULL;
        DOMElement*     resourceAttrValue = NULL;
        DOMAttr*        resourceAttributeId = NULL;
        DOMAttr*        resourceDatatype = NULL;
        
        resource = myDoc->createElementNS(XMLString::transcode(xmlns.c_str()), XMLString::transcode("Resource"));
        resourceAttr = myDoc->createElementNS(XMLString::transcode(xmlns.c_str()), XMLString::transcode("Attribute"));
        resourceAttrValue = myDoc->createElementNS(XMLString::transcode(xmlns.c_str()), XMLString::transcode("AttributeValue"));
        resourceAttributeId = myDoc->createAttribute(XMLString::transcode("AttributeId"));        
        resourceDatatype = myDoc->createAttribute(XMLString::transcode("DataType")); 
        resourceDatatype->setValue(XMLString::transcode(datatype.c_str()));
        resourceAttributeId->setValue(XMLString::transcode(resourceAttribute.c_str()));
        resourceText = myDoc->createTextNode(XMLString::transcode(_resource.c_str()));  
        resourceAttr->setAttributeNode(resourceAttributeId);
        resourceAttr->setAttributeNode(resourceDatatype);
        resourceAttrValue->appendChild(resourceText);
        resourceAttr->appendChild(resourceAttrValue);
        resource->appendChild(resourceAttr);
        
        /*action*/
        DOMText*        actionText = NULL;
        DOMElement*     actionAttr = NULL;
        DOMElement*     actionAttrValue = NULL;
        DOMAttr*        actionAttributeId = NULL;
        DOMAttr*        actionDatatype = NULL;
        
        action = myDoc->createElementNS(XMLString::transcode(xmlns.c_str()), XMLString::transcode("Action"));
        actionAttr = myDoc->createElementNS(XMLString::transcode(xmlns.c_str()), XMLString::transcode("Attribute"));
        actionAttrValue = myDoc->createElementNS(XMLString::transcode(xmlns.c_str()), XMLString::transcode("AttributeValue"));
        actionAttributeId = myDoc->createAttribute(XMLString::transcode("AttributeId"));        
        actionDatatype = myDoc->createAttribute(XMLString::transcode("DataType")); 
        actionDatatype->setValue(XMLString::transcode(datatype.c_str()));
        actionAttributeId->setValue(XMLString::transcode(actionAttribute.c_str()));
        actionText = myDoc->createTextNode(XMLString::transcode(_action.c_str())); 
        actionAttr->setAttributeNode(actionAttributeId);
        actionAttr->setAttributeNode(actionDatatype);
        actionAttrValue->appendChild(actionText);
        actionAttr->appendChild(actionAttrValue);
        action->appendChild(actionAttr);
        
        /*action*/
        DOMText*        environmentText = NULL;
        DOMElement*     environmentAttr = NULL;
        DOMElement*     environmentAttrValue = NULL;
        DOMAttr*        environmentAttributeId = NULL;
        DOMAttr*        environmentDatatype = NULL;
        
        environment = myDoc->createElementNS(XMLString::transcode(xmlns.c_str()), XMLString::transcode("Environment"));
                
        /*
         * 
         * Attaching all the nodes
         * 
         */
        syslog(LOG_INFO, " - Attaching");
        root->appendChild(subject);        
        root->appendChild(resource);
        root->appendChild(action);        
        root->appendChild(environment); 
         if(_conf->isDebug()) {
                syslog(LOG_DEBUG, "Request: %s", outputXmlStream(myDoc).c_str());
         }
        xmlResult = outputXmlStream(myDoc);
	//std::string xmlResultTmp = "request=";
	std::string xmlResultTmp = "";
	xmlResultTmp += xmlResult;
	xmlResult = xmlResultTmp;

        
    } catch (const DOMException& e) {
        syslog(LOG_ERR, " - Error while molding XML Request: %s", XMLString::transcode(e.getMessage()));
    }
    
    /*
     * Cleaning up
     */
    myDoc->release();
    XMLPlatformUtils::Terminate();
    
    return xmlResult;
}

/**
* 
* @param subject
* @param action
* @param resource
* @return 
*/
RestClient::response pdpSOAPRequester::restRequest(std::string subject, std::string action, std::string resource)
{
    std::string request = "";   
    
    /**
    *
    *	REST variables
    *
    **/
    RestClient client;
    RestClient::response res;
    
    request += "user=";
    request += subject;
    request += "&resource=";
    request += resource;
    request += "&action=";
    request += action;
    request = buildXmlRequest(subject, resource, action);
    if(_conf->isDebug())
    {
        syslog(LOG_INFO, " - REST Call.....\n"); 
        syslog(LOG_INFO, "Request: %s", request.c_str());
    }

    //res = client.post(_conf->getPdpEndpoint(), "application/x-www-form-urlencoded", request);
    res = client.post(_conf->getPdpEndpoint(), "application/xml", request);
    if (_conf->isDebug())
    {
        syslog(LOG_INFO, " - Status code: %d", res.code);
        syslog(LOG_INFO, " - Body: %s", res.body.c_str());
    }
   
    return res;
}

//int pdpSOAPRequester::soapRequest(std::string subject, std::string action, std::string resource, bool _SAML, ns11__XACMLAuthzDecisionResponse* response, struct soap* soap)
//{
//    if (_conf->isDebug())
//		syslog(LOG_INFO, " - Using gSoap");
//	int result = 0;
//        bool SAML = _SAML;
//
//	const char* endpoint = _conf->getPdpEndpoint();
//
//	const char* soapAction = _conf->getApplicationName();
//	const char* soapSubject= subject.c_str();
//	const char* soapResource = resource.c_str();
//	const char* soapEnvironment = (char*)"";
//
//	action = _conf->getApplicationName();
//
//	/**
//	*
//	*
//	*	Initialisation
//	*
//	**/
//	if(_conf->isDebug())
//		syslog(LOG_INFO," - Soap initialisation\n");
//
//	/**
//	*
//	*   On  déclare et  initialise  la  requête
//	*
//	**/
//	void * query;
//	
//	if (SAML == true)
//	{
//		ns11__SAML2XACMLAuthzDecisionQuery *queryTmp = soap_new_ns11__SAML2XACMLAuthzDecisionQuery(soap,-1);
//		query= queryTmp;
//	}
//	else
//	{
//		ns11__XACMLAuthzDecision *queryTmp = soap_new_ns11__XACMLAuthzDecision(soap,-1);
//		query= queryTmp;
//	}
//	ns5__RequestType *requestType=soap_new_ns5__RequestType(soap,-1);
//	ns3__XACMLAuthzDecisionQueryType *xacmlauthzdecisionqueryType=soap_new_ns3__XACMLAuthzDecisionQueryType(soap,-1);
//
//	/**
//	*
//	*   On  déclare et  initialise  l'attribut  Action
//	*
//	**/
//	ns5__ActionType *actionType=soap_new_ns5__ActionType(soap,-1);
//	ns5__AttributeType  *actionAttributeType=soap_new_ns5__AttributeType(soap,-1);
//	ns5__AttributeValueType *actionAttributeValueType=soap_new_ns5__AttributeValueType(soap,-1);
//	
//	actionAttributeValueType->__anyAttribute=(char*)soapAction;
//	actionAttributeValueType->__item=(char*)soapAction;
//	actionAttributeValueType->__mixed=(char*) soap_malloc(soap, strlen(soapAction)+1);
//	actionAttributeValueType->__mixed=(char*)soapAction;
//	actionAttributeType->DataType=(char*)XACML_DATATYPE_STRING;
//	actionAttributeType->Issuer=new std::string("Issuer");
//	actionAttributeType->AttributeId=(char*)"urn:oasis:names:tc:xacml:1.0:action:action-id";
//
//	if (SAML == true)
//	{
//		actionAttributeType->AttributeValue.push_back(actionAttributeValueType);
//		((ns11__SAML2XACMLAuthzDecisionQuery*)(query))->xacmlauthzdecisionquery=xacmlauthzdecisionqueryType;
//		((ns11__SAML2XACMLAuthzDecisionQuery*)(query))->xacmlauthzdecisionquery->ns5__Request=requestType;
//		((ns11__SAML2XACMLAuthzDecisionQuery*)(query))->xacmlauthzdecisionquery->ns5__Request->Action=actionType;
//		((ns11__SAML2XACMLAuthzDecisionQuery*)(query))->xacmlauthzdecisionquery->ns5__Request->Action->Attribute.push_back(actionAttributeType);
//	}
//	else
//	{
//	    ((ns11__XACMLAuthzDecision*)(query))->xacmlRequest=requestType;
//		actionAttributeType->AttributeValue.push_back(actionAttributeValueType);
//		((ns11__XACMLAuthzDecision*)(query))->xacmlRequest->Action=actionType;
//	    ((ns11__XACMLAuthzDecision*)(query))->xacmlRequest->Action->Attribute.push_back(actionAttributeType);
//	}
//
//	/**
//	*
//	*   On  déclare et  initialise  l'attribut  Subject
//	*
//	**/
//	ns5__SubjectType *subjectType=soap_new_ns5__SubjectType(soap,-1);
//	ns5__AttributeType  *subjectAttributeType=soap_new_ns5__AttributeType(soap,-1);
//	ns5__AttributeValueType *subjectAttributeValueType=soap_new_ns5__AttributeValueType(soap,-1);
//	
//	subjectAttributeValueType->__anyAttribute=(char*)soapSubject;
//	subjectAttributeValueType->__item=(char*)soapSubject;
//	subjectAttributeValueType->__mixed=(char*) soap_malloc(soap, strlen(soapSubject)+1);
//	subjectAttributeValueType->__mixed=(char*)soapSubject;
//	subjectAttributeType->DataType=(char*)XACML_DATATYPE_STRING;
//	subjectAttributeType->Issuer=new std::string("Issuer");
//	subjectAttributeType->AttributeId=(char*)"urn:oasis:names:tc:xacml:1.0:subject:subject-id";
//
//	if (SAML == true)
//	{
//		subjectAttributeType->AttributeValue.push_back(subjectAttributeValueType);
//		subjectType->Attribute.push_back(subjectAttributeType);
//		((ns11__SAML2XACMLAuthzDecisionQuery*)(query))->xacmlauthzdecisionquery->ns5__Request->Subject.push_back(subjectType);
//	}
//	else
//	{
//		subjectAttributeType->AttributeValue.push_back(subjectAttributeValueType);
//		subjectType->Attribute.push_back(subjectAttributeType);
//		((ns11__XACMLAuthzDecision*)(query))->xacmlRequest->Subject.push_back(subjectType);		
//	}
//
//	/**
//	*
//	*   On  déclare et  initialise  l'attribut  Ressource
//	*
//	**/
//	ns5__ResourceType *resourceType=soap_new_ns5__ResourceType(soap,-1);
//	ns5__AttributeType  *resourceAttributeType=soap_new_ns5__AttributeType(soap,-1);
//	ns5__AttributeValueType *resourceAttributeValueType=soap_new_ns5__AttributeValueType(soap,-1);
//	
//	resourceAttributeValueType->__anyAttribute=(char*)soapResource;
//	resourceAttributeValueType->__item=(char*)soapResource;
//	resourceAttributeValueType->__mixed=(char*) soap_malloc(soap, strlen(soapResource)+1);
//	resourceAttributeValueType->__mixed=(char*)soapResource;
//	resourceAttributeType->DataType=(char*)XACML_DATATYPE_STRING;
//	resourceAttributeType->Issuer=new std::string("Issuer");
//	resourceAttributeType->AttributeId=(char*)"urn:oasis:names:tc:xacml:1.0:resource:resource-id";
//	if (SAML == true)
//	{
//		resourceAttributeType->AttributeValue.push_back(resourceAttributeValueType);
//		resourceType->Attribute.push_back(resourceAttributeType);
//		((ns11__SAML2XACMLAuthzDecisionQuery*)(query))->xacmlauthzdecisionquery->ns5__Request->Resource.push_back(resourceType);
//		((ns11__SAML2XACMLAuthzDecisionQuery*)(query))->xacmlauthzdecisionquery->IssueInstant=time(NULL);
//		((ns11__SAML2XACMLAuthzDecisionQuery*)(query))->xacmlauthzdecisionquery->Version="2.0";   
//		((ns11__SAML2XACMLAuthzDecisionQuery*)(query))->xacmlauthzdecisionquery->ID="ID-";
//	}
//	else
//	{
//	 	resourceAttributeType->AttributeValue.push_back(resourceAttributeValueType);
//		resourceType->Attribute.push_back(resourceAttributeType);
//		((ns11__XACMLAuthzDecision*)(query))->xacmlRequest->Resource.push_back(resourceType);
//	}
//
//	/**
//	*
//	*   On  déclare et  initialise  l'attribut  Environment
//	*
//	**/
//	ns5__EnvironmentType *environmentType=soap_new_ns5__EnvironmentType(soap,-1);
//	ns5__AttributeType  *environmentAttributeType=soap_new_ns5__AttributeType(soap,-1);
//	ns5__AttributeValueType *environmentAttributeValueType=soap_new_ns5__AttributeValueType(soap,-1);
//	
//	environmentAttributeValueType->__anyAttribute=(char*)soapEnvironment;
//	environmentAttributeValueType->__item=(char*)soapEnvironment;
//	environmentAttributeValueType->__mixed=(char*) soap_malloc(soap, 20);
//	environmentAttributeValueType->__mixed=(char*)soapEnvironment;
//	environmentAttributeType->DataType=(char*)XACML_DATATYPE_STRING;
//	environmentAttributeType->Issuer=new std::string("Issuer");
//	environmentAttributeType->AttributeId=(char*)"urn:oasis:names:tc:xacml:1.0:environment:environment-id";
//	if (SAML == true)
//	{
//		environmentAttributeType->AttributeValue.push_back(environmentAttributeValueType);
//    		((ns11__SAML2XACMLAuthzDecisionQuery*)(query))->xacmlauthzdecisionquery->ns5__Request->Environment=environmentType;
//	    	((ns11__SAML2XACMLAuthzDecisionQuery*)(query))->xacmlauthzdecisionquery->ns5__Request->Environment->Attribute.push_back(environmentAttributeType);
//	}
//	else
//	{
//        	environmentAttributeType->AttributeValue.push_back(environmentAttributeValueType);
//	        ((ns11__XACMLAuthzDecision*)(query))->xacmlRequest->Environment=environmentType;
//        	((ns11__XACMLAuthzDecision*)(query))->xacmlRequest->Environment->Attribute.push_back(environmentAttributeType);
//	}
//        if (SAML)
//                result = soap_call___ns1__SAML2XACMLAuthzDecisionQuery(soap, endpoint, action.c_str(), ((ns11__SAML2XACMLAuthzDecisionQuery*)(query)), response);
//        else
//                result = soap_call___ns1__XACMLAuthzDecision(soap, endpoint, action.c_str(), (ns11__XACMLAuthzDecision*)query, response);
//        
//        return result;
//}

/**
 * XML parsing of the response given by the PDP
 * @param response : response given from the PDP
 * @param decision : decision after parsing
 * @param statusMsg : status message 
 * @param statusCode : status code
 * @param obligations : list of obligations received 
 */
void pdpSOAPRequester::xmlRestParseResponse(std::string response, std::string decision, std::string statusMsg, std::string statusCode, std::string *obligations)
{    
    try {
        XMLPlatformUtils::Initialize();
        
    } catch (const XMLException& e) {
        syslog(LOG_ERR, "Error during DOM initilization");
        exit(1);
    }
    std::string xmlStringTmp = response;
    MemBufInputSource myXmlBuf((const XMLByte*)response.c_str(), xmlStringTmp.length(), "xmlStringTmp (in memory)", false);    
    
    XercesDOMParser *parser = new XercesDOMParser();
    
    /* Parser Initilization */    
    parser->setValidationScheme(XercesDOMParser::Val_Never);
    parser->setDoNamespaces(false);
    parser->setDoSchema(false);
    parser->setLoadExternalDTD(false);    
    
    try {

        parser->parse(myXmlBuf);                
        DOMDocument* myDoc = parser->getDocument();
        DOMElement* elementRoot = myDoc->getDocumentElement();
        if (!elementRoot)
            throw(std::runtime_error("empty XML document"));
        DOMNodeList* children = elementRoot->getChildNodes();

        /*
         * 
         * Recherche dans l'arbre
         * 
         */
        // Fetching Decision
        DOMNodeList *decisionXml = myDoc->getElementsByTagName(XMLString::transcode("Decision"));        
        if (decisionXml->getLength() > 0)
        {
            decision = XMLString::transcode(decisionXml->item(0)->getTextContent());
            if (_conf->isDebug())
                    syslog(LOG_INFO, " - Decision: %s", decision.c_str());
        }
        
        // Fetching Status Code
        DOMNodeList *statusCodeXml = myDoc->getElementsByTagName(XMLString::transcode("StatusCode"));
        if (statusCodeXml->getLength() > 0)
        {
            DOMNode *statusCodeNodeXml = statusCodeXml->item(0);
            DOMElement *statusCodeElementXml = dynamic_cast<DOMElement* >(statusCodeNodeXml);
            statusCode = XMLString::transcode(statusCodeElementXml->getAttribute(XMLString::transcode("Value")));        
            if (_conf->isDebug())
                    syslog(LOG_INFO, " - Status Code: %s", statusCode.c_str());
        }
        // Fetching Status Message
        DOMNodeList *statusMsgXml = myDoc->getElementsByTagName(XMLString::transcode("StatusMessage"));        
        if (statusMsgXml->getLength() > 0)
        {
            statusMsg = XMLString::transcode(statusMsgXml->item(0)->getTextContent());
            if (_conf->isDebug())
                    syslog(LOG_INFO, " - Status Message: %s", statusMsg.c_str());
        }
        
        // TODO: Parse and process Obligations

    } catch (XMLException &e)
    {
        syslog(LOG_ERR, "Error while parsing response: %s", XMLString::transcode(e.getMessage()));
    }  
    
    delete parser;
    try {
//        xercesc_3_1::XMLPlatformUtils::Terminate();
    } catch (const XMLException& e) {
        syslog(LOG_ERR, "Error during DOM desallocation");
        exit(1);
    }
}

/**
 *
 * Main part of this class, get the PDP Decision and return it to pam_handler. 
 * Deny if something wrong happen, XACML_DECISION if everything works fine
 *
 */
char* pdpSOAPRequester::getPdpDecision(const char* xacmlRequest)
{

	/////////////////////////////////////////////////////////////////////////
	/// TALK TO THE POLICY DECISION POINT
	/////////////////////////////////////////////////////////////////////////
//	char* xacmlResponse = NULL;
//	unsigned int decision;
//	bool SAML = false;
//	bool REST = false;
	RestClient::response responseXml;

        if (_conf->isDebug()) {
                syslog(LOG_INFO, " - REST PDP");
        }
//	REST = true;	

	std::string xacmlResponseTmp = "";

	if (_conf->isUseSSL())
	{
		/**
		 *
		 * SSL Parts...
		 *
		 */
		// TODO: implement the SSL part
		// ...
	}
	if (_conf->isDebug())
		syslog(LOG_INFO, " - Initialisation request");

	char hostname[128];
	char domain[256];
	char * pch;

	std::string subject = _conf->getUsername();
	if (subject == "Deny")
	{
		syslog(LOG_ERR, " - Permission Denied");
		return (char*)"Deny";
	}

	//std::string resource = "cn=";
	std::string resource = "";
	//TODO: Check for null value
	gethostname(hostname, 127);
	resource += hostname;
	//resource += ",";
	//TODO: Check for null value
	//if (_conf->getOu())
	//	resource += _conf->getOu();
	//TODO: Check for null value
//	if(getdomainname(domain, 255) == 0)
//	{
//		if (domain)
//		{
//			pch = strtok (domain,".");
//			while (pch != NULL)
//			{
//				resource += ",dc=";
//				resource += pch;
//				pch = strtok (NULL, ".");
//			}
//		}
//	}	
//	else
//		syslog(LOG_ERR, "getdomain error");
	if (_conf->isDebug())
		syslog(LOG_INFO, " - Resource: %s\n", resource.c_str());

//	struct soap *soap;
//        int result = 0;
//	const char* endpoint = _conf->getPdpEndpoint();
	const char* action = "";
	action = _conf->getApplicationName();
//        soap = soap_new();
//        ns11__XACMLAuthzDecisionResponse* response = soap_new_ns11__XACMLAuthzDecisionResponse(soap,-1);
        
        if (_conf->isDebug())
		syslog(LOG_INFO, " - Initilization Ok, moving on to calls");

	/**
	*
	*	Call
	*
	**/	
	
        responseXml = restRequest(subject, action, resource);           
	
	if (responseXml.code == 200) {
            syslog(LOG_INFO," - REST Call went well.....\n");	
	}
	else
	{
            syslog(LOG_ERR, " - Something went wrong; here is the backtrace:");		
            syslog(LOG_ERR, " - %s", responseXml.body.c_str());
            syslog(LOG_ERR, " - Denying access");
            //NOTE: Pour la version finale
            return (char*)"XACML_DECISION_Deny";
	}

	/**
	*
	*	Désallocation
	*
	**/
        // TODO: MemCheck
	
	/**
	*
	*   Maintenant on parse pour avoir la réponse
         * TODO: Parser vraiment le XML avec Xerces-c
	*
	**/

        std::string finalDecision, statusMsg;
        std::string *obligations;
        std::string statusCode;

        xmlRestParseResponse(responseXml.body, finalDecision, statusMsg, statusCode, obligations);
        if (_conf->isDebug()) {
                syslog(LOG_INFO, " - Parsing response for decision");
        }
        if (strstr(responseXml.body.c_str(), "NOT_APPLICABLE"))
        {
                if (_conf->isDebug())
                        syslog(LOG_INFO, " - NOT APPLICABLE");
                return (char*)"XACML_DECISION_NotApplicable";
        }
        else if (strstr(responseXml.body.c_str(), "INDETERMINATE"))
        {
                if (_conf->isDebug())
                        syslog(LOG_INFO, " - INDETERMINATE");
                return (char*)"XACML_DECISION_Indeterminate";
        }
        else if (strstr(responseXml.body.c_str(), "PERMIT"))
        {
                if (_conf->isDebug())
                        syslog(LOG_INFO, " - PERMIT");
                return (char*)"XACML_DECISION_Permit";
        }
        else	
        {
                if (_conf->isDebug())
                        syslog(LOG_INFO, " - DENY");
                return (char*)"XACML_DECISION_Deny";
        }
	
        
	
	return (char*)"XACML_DECISION_Deny";
}
}
