/********************************************************************************
*                                                                               *
* Copyright (c) 2007, Computer Networks and Internet, University of Tübingen,   *
* Sand 13, 72076 Tübingen, Germany                                              *
*                                                                               *
* All rights reserved.                                                          *
*                                                                               *
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Lesser General Public                    *
* License as published by the Free Software Foundation; either                  *
* version 2.1 of the License, or (at your option) any later version.            *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU              *
* Lesser General Public License for more details.                               *
*                                                                               *
* You should have received a copy of the GNU Lesser General Public              *
* License along with this library; if not, write to the Free Software           *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA       *
*                                                                               *
********************************************************************************/
/*
 * AttributeProvider.cpp
 *
 *  Created on: Jul 2, 2008
 *      Author: theide
 */
#include <strings.h>
#include "AttributeProvider.h"

namespace pam_xacml {

AttributeProvider::AttributeProvider(char* username, char* appName) {
	_username = std::string(username);
	_appname = std::string(appName);

	char hostname[128];
	gethostname(hostname, 127);
	_hostname = std::string(hostname);
}

AttributeProvider::~AttributeProvider() {
	// TODO Auto-generated destructor stub
}

std::string AttributeProvider::getTimeZone()
{
	char buf[128];
	time_t curtime;
	time(&curtime);
	struct tm* tm_gmt;
	struct tm* tm_local;

	tzset();

	tm_gmt = gmtime(&curtime);
	tm_local = localtime(&curtime);

	int hours = tm_local->tm_hour - tm_gmt->tm_hour;
	int minutes = tm_local->tm_min - tm_gmt->tm_min;

	snprintf(buf, 127, "%+02d:%02d", hours, minutes);
	return std::string(buf);
}

std::string AttributeProvider::getDate()
{
	char buf[128];
	time_t loctime;
	time(&loctime);
	strftime(buf, 128, "%Y-%m-%d", localtime(&loctime));
	return std::string(buf) + getTimeZone();
}
std::string AttributeProvider::getTime()
{
	char buf[128];
	time_t loctime;
	time(&loctime);
	strftime(buf, 128, "%H:%M:%S", localtime(&loctime));
	return std::string(buf) + getTimeZone();
}

std::string AttributeProvider::getValue(std::string fieldname)
{
	if (0==fieldname.compare("Resource")) {
		return _appname;
	}
	if (0==fieldname.compare("Username")) {
		return _username;
	}
	if (0==fieldname.compare("Action")) {
		return std::string("PamXacml Authorization");
	}
	if (0==fieldname.compare("Hostname")) {
		return _hostname;
	}
	if (0==fieldname.compare("SystemDate")) {
		return getDate();
	}
	if (0==fieldname.compare("SystemTime")) {
		return getTime();
	}
	throw std::runtime_error((std::string("AttributeProvider has no value for field - this propably is a typo in your request template: ") + std::string(fieldname)).c_str());
}

std::string AttributeProvider::valueFromApplication(std::string application)
{
	FILE* app;
	std::string returnValue;
	char buf[513];
	app = popen(application.c_str(), "r");
	if (app == NULL)
		throw std::runtime_error("Could not invoke application that was specified in the request template.");

	while(!feof(app)) {
		bzero(buf, 513);
		fread(buf, 512, 1, app);
		returnValue += buf;
	}

	pclose(app);

	return returnValue.c_str();
}


}
