/********************************************************************************
*                                                                               *
* Copyright (c) 2007, Computer Networks and Internet, University of Tübingen,   *
* Sand 13, 72076 Tübingen, Germany                                              *
*                                                                               *
* All rights reserved.                                                          *
*                                                                               *
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Lesser General Public                    *
* License as published by the Free Software Foundation; either                  *
* version 2.1 of the License, or (at your option) any later version.            *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU              *
* Lesser General Public License for more details.                               *
*                                                                               *
* You should have received a copy of the GNU Lesser General Public              *
* License along with this library; if not, write to the Free Software           *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA       *
*                                                                               *
********************************************************************************/

#include "PAMRequestBuilder.h"
#include "AttributeProvider.h"

// Standard C includes
#include <stdlib.h>
#include <stdio.h>
#include <sys/stat.h>
#include <string.h>


#include <syslog.h>

// C++ includes
#include <sstream>
#include <fstream>
#include <stdexcept>
#include <new>

// pam_xacml includes
#include "utils.h"

#define PAMXACML_NAMESPACE "pamxacml"

namespace pam_xacml {


/*****************************************************************************
 *** Template based Authz Request with local knowledge                     ***
 *****************************************************************************/

PAMRequestBuilder::PAMRequestBuilder(AttributeProvider* provider, PAMConfiguration* conf) {
	_provider = provider;
	_conf = conf;
}


const char* PAMRequestBuilder::buildAuthzRequest() {
	return fillOutTemplate();
}

const char* PAMRequestBuilder::fillOutTemplate()
{
	char line[512];
	std::string file;
	std::string fieldName;
	std::string request;
	std::string::size_type lastOccurence = 0;
	std::string::size_type startIndex = 0;
	std::string::size_type endIndex = 0;

	FILE* fh;

	fh = fopen(this->_conf->getRequestTemplate(), "r");
	if (!fh)
		throw std::runtime_error("Could not open request template file.");

	while(!feof(fh)) {
		bzero(line,512);
		fread(line, 1, 512-1, fh);
		file.append(line);
	}
	fclose(fh);

	while (std::string::npos != (startIndex = file.find("<" PAMXACML_NAMESPACE ":", lastOccurence))) {
		endIndex = file.find("/>", startIndex);
		fieldName = file.substr(startIndex+strlen(PAMXACML_NAMESPACE) +2, endIndex-startIndex-strlen(PAMXACML_NAMESPACE)-3);
		//printf("  * Found namespace declaration at (last=%d): %d  --- FIELD = **%s**\n", lastOccurence,startIndex,fieldName.c_str());
		if (std::string::npos == fieldName.find("INVOKE", 0)) {
			file.replace(startIndex, endIndex-startIndex+2, _provider->getValue(fieldName));
		}
		lastOccurence = startIndex+2;
	}

	lastOccurence=0;
	while (std::string::npos != (startIndex = file.find("<" PAMXACML_NAMESPACE ":INVOKE>", lastOccurence))) {
		endIndex = file.find("</" PAMXACML_NAMESPACE ":INVOKE>", startIndex);
		fieldName = file.substr(startIndex+strlen(PAMXACML_NAMESPACE) +9, endIndex-startIndex-strlen(PAMXACML_NAMESPACE)-9);			//printf("  * Found namespace declaration at (last=%d): %d  --- FIELD = **%s**\n", lastOccurence,startIndex,fieldName.c_str());
		//throw std::runtime_error((std::string("Fieldname: ") + std::string(fieldName)).c_str());
		file.replace(startIndex, endIndex-startIndex+18, _provider->valueFromApplication(fieldName));
		lastOccurence = startIndex+2;
	}


	return strdup(file.c_str());
}


}
