/********************************************************************************
*                                                                               *
* Copyright (c) 2007, Computer Networks and Internet, University of Tübingen,   *
* Sand 13, 72076 Tübingen, Germany                                              *
*                                                                               *
* All rights reserved.                                                          *
*                                                                               *
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Lesser General Public                    *
* License as published by the Free Software Foundation; either                  *
* version 2.1 of the License, or (at your option) any later version.            *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU              *
* Lesser General Public License for more details.                               *
*                                                                               *
* You should have received a copy of the GNU Lesser General Public              *
* License along with this library; if not, write to the Free Software           *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA       *
*                                                                               *
********************************************************************************/
#include "utils.h"

// Standard C includes
#include <stdlib.h>
#include <sys/stat.h>
#include <stdio.h>
#include <syslog.h>

//C++ includes
#include <stdexcept>


namespace pam_xacml {

/**
 * Read file into buffer that is allocated internally. The caller is responsible for deallocating the memory!
 * @param filename is the name of the file
 * @return nul terminated buffer that contains whole file content
 */
char* readFileIntoBuffer(const char* filename)
{
	char* buf;
	struct stat file_info;
	
	FILE* fh = fopen(filename, "r");
	if (!fh) {
    	throw std::invalid_argument("readFileIntoBuffer Could not open file with name " + std::string(filename));
	}
	
	fstat(fileno(fh), &file_info);
	if (S_ISDIR(file_info.st_mode)) {
		fclose(fh);
		throw std::invalid_argument("readFileIntoBuffer " + std::string(filename)+" is a directory!");
	}
	
	buf = (char*)malloc(file_info.st_size + 3);
	if (buf != NULL) {
		fread(buf, 1, file_info.st_size, fh);
		fclose(fh);
		buf[file_info.st_size] = '\0';
		syslog(LOG_INFO, "  readFileIntoBuffer Success reading file content: %s", buf);
		return buf;
	} else {
		fclose(fh);
		throw std::bad_alloc();
	}
}

}
