/********************************************************************************
*                                                                               *
* Copyright (c) 2007, Computer Networks and Internet, University of Tübingen,   *
* Sand 13, 72076 Tübingen, Germany                                              *
*                                                                               *
* All rights reserved.                                                          *
*                                                                               *
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Lesser General Public                    *
* License as published by the Free Software Foundation; either                  *
* version 2.1 of the License, or (at your option) any later version.            *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU              *
* Lesser General Public License for more details.                               *
*                                                                               *
* You should have received a copy of the GNU Lesser General Public              *
* License along with this library; if not, write to the Free Software           *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA       *
*                                                                               *
********************************************************************************/

#ifndef XACML_H_
#define XACML_H_

#define XACML_PERMIT 1
#define XACML_DENY 2
#define XACML_INDETERMINATE 3
#define XACML_NOT_APPLICABLE 4
#define PAM_XACML_DENY 5

#include <string.h>
#include <libxml/xpath.h>

namespace pam_xacml { 

/*****************************************************************************
 *** FUNCTIONS FOR XML and DOM  (for internal use)                         ***
 *****************************************************************************/
class XMLDocument {
	public:
		XMLDocument(const char* xmlString);
		virtual ~XMLDocument();

		// beware of the namespaces in XML!
		int executeXpathExpression(const xmlChar* xpathExpr, xmlXPathObjectPtr* xpathObj);

		static int getFirstTextNode(const xmlNodePtr node, char **text);
		static xmlNodePtr searchNodeByName(const xmlNodePtr node, const char* name);
	
		
		xmlDocPtr getDoc() { return _doc; };
		
	protected:
		xmlDocPtr _doc;
};


/*****************************************************************************
 *** FUNCTIONS FOR XACML	                                               ***
 *****************************************************************************/
class XACMLReply {
	public:
		XACMLReply(XMLDocument *xmlDocument);
		virtual ~XACMLReply() {};

		//return true if it has obligations otherwise false
		bool hasObligations(); 

		//return decision, set statusNode to NULL if no status is required
		int getDecision(); //throws std::runtime_error

		//return decision, set statusNode to NULL if no status is required
		xmlNodePtr getStatus(); 
		
		XMLDocument* getXMLDocument() { return _xmlDocument; }

	protected:
		XMLDocument* _xmlDocument;
};


}
#endif

