/********************************************************************************
*                                                                               *
* Copyright (c) 2007, Computer Networks and Internet, University of Tübingen,   *
* Sand 13, 72076 Tübingen, Germany                                              *
*                                                                               *
* All rights reserved.                                                          *
*                                                                               *
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Lesser General Public                    *
* License as published by the Free Software Foundation; either                  *
* version 2.1 of the License, or (at your option) any later version.            *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU              *
* Lesser General Public License for more details.                               *
*                                                                               *
* You should have received a copy of the GNU Lesser General Public              *
* License along with this library; if not, write to the Free Software           *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA       *
*                                                                               *
********************************************************************************/
/*
 * XacmlRequestDocument.cpp
 *
 *  Created on: 17.07.2008
 *      Author: theide
 */

#include "XacmlRequestDocument.h"

namespace xacml {

	XacmlRequestDocument::XacmlRequestDocument() {
		// TODO Auto-generated constructor stub

	}

	XacmlRequestDocument::~XacmlRequestDocument() {
		// TODO Auto-generated destructor stub
	}

	void XacmlRequestDocument::addSubjectAttributes(vector<XacmlAttribute*> attr)
	{
		for (vector<XacmlAttribute*>::iterator i = attr.begin(); i != attr.end(); i++)
			subjectAttributes.push_back(*i);
	}

	void XacmlRequestDocument::addResourceAttributes(vector<XacmlAttribute*> attr)
	{
		for (vector<XacmlAttribute*>::iterator i = attr.begin(); i != attr.end(); i++)
			resourceAttributes.push_back(*i);
	}

	void XacmlRequestDocument::addActionAttributes(vector<XacmlAttribute*> attr)
	{
		for (vector<XacmlAttribute*>::iterator i = attr.begin(); i != attr.end(); i++)
			actionAttributes.push_back(*i);
	}

    void XacmlRequestDocument::addEnvironmentAttributes(vector<XacmlAttribute*> attr)
    {
    	for (vector<XacmlAttribute*>::iterator i = attr.begin(); i != attr.end(); i++)
    		environmentAttributes.push_back(*i);
    }

    void XacmlRequestDocument::setSubjectCategory(string cat)
    {
    	subjectCategory = cat;
    }

    string XacmlRequestDocument::getSubjectCategory()
    {
    	return subjectCategory;
    }

    string XacmlRequestDocument::getPrimarySubjectID()
    {
    	// return first match on subject id urn - this is a hack for besters pdp

    	for (vector<XacmlAttribute*>::iterator i = this->subjectAttributes.begin(); i != this->subjectAttributes.end(); i++) {
    		if ((*i)->getName() == string(SUBJECT_ID_URN)) {
    			return *((*i)->getValueIterator());
    		}
    	}
    }

    vector<XacmlAttribute*>::iterator XacmlRequestDocument::getAttributeIteratorBegin(XacmlSection sect) {
    	switch (sect) {
    	case SUBJECT:
    		return this->subjectAttributes.begin();
    	case RESOURCE:
    	    return this->resourceAttributes.begin();
    	case ENVIRONMENT:
    		return this->environmentAttributes.begin();
    	case ACTION:
    		return this->actionAttributes.begin();
    	default:
    		throw runtime_error(__FILE__ "getAttributeIteratorBegin: invalid XacmlSection.");
    	}
    }

    vector<XacmlAttribute*>::iterator XacmlRequestDocument::getAttributeIteratorEnd(XacmlSection sect) {
    	switch (sect) {
    	case SUBJECT:
    		return this->subjectAttributes.end();
    	case RESOURCE:
    	    return this->resourceAttributes.end();
    	case ENVIRONMENT:
    		return this->environmentAttributes.end();
    	case ACTION:
    		return this->actionAttributes.end();
    	default:
    		throw runtime_error(__FILE__ "getAttributeIteratorEnd: invalid XacmlSection.");
    	}
    }


    string XacmlRequestDocument::dump()
    {
    	string rv;
    	for (vector<XacmlAttribute*>::iterator i = this->subjectAttributes.begin(); i != this->subjectAttributes.end(); i++) {
    		rv += "SubjectAttr(" + (*i)->getName() + ") of type (" + (*i)->getType() + ")\n";
			for (vector<string>::iterator j = (*i)->getValueIterator(); j != (*i)->getValueIteratorEnd(); j++)
				rv += "\tValue: " + (*j) + "\n";
    	}
    	rv += "SubjectCategory=" + this->getSubjectCategory() + "\n";
    	for (vector<XacmlAttribute*>::iterator i = this->resourceAttributes.begin(); i != this->resourceAttributes.end(); i++) {
    	    rv += "ResourceAttr(" + (*i)->getName() + ") of type (" + (*i)->getType() + ")\n";
    	    for (vector<string>::iterator j = (*i)->getValueIterator(); j != (*i)->getValueIteratorEnd(); j++)
    			rv += "\tValue: " + (*j) + "\n";
    	}
    	for (vector<XacmlAttribute*>::iterator i = this->actionAttributes.begin(); i != this->actionAttributes.end(); i++) {
    	    rv += "ActionAttr(" + (*i)->getName() + ") of type (" + (*i)->getType() + ")\n";
    	    for (vector<string>::iterator j = (*i)->getValueIterator(); j != (*i)->getValueIteratorEnd(); j++)
    			rv += "\tValue: " + (*j) + "\n";
		}
    	for (vector<XacmlAttribute*>::iterator i = this->environmentAttributes.begin(); i != this->environmentAttributes.end(); i++) {
    		rv += "EnvorinmentAttr(" + (*i)->getName() + ") of type (" + (*i)->getType() + ")\n";
    		for (vector<string>::iterator j = (*i)->getValueIterator(); j != (*i)->getValueIteratorEnd(); j++)
    			rv += "\tValue: " + (*j) + "\n";
    	}

    	return rv;
    }

}
