/********************************************************************************
*                                                                               *
* Copyright (c) 2007, Computer Networks and Internet, University of Tübingen,   *
* Sand 13, 72076 Tübingen, Germany                                              *
*                                                                               *
* All rights reserved.                                                          *
*                                                                               *
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Lesser General Public                    *
* License as published by the Free Software Foundation; either                  *
* version 2.1 of the License, or (at your option) any later version.            *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU              *
* Lesser General Public License for more details.                               *
*                                                                               *
* You should have received a copy of the GNU Lesser General Public              *
* License along with this library; if not, write to the Free Software           *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA       *
*                                                                               *
********************************************************************************/
/*
 * XacmlRequestDocument.h
 *
 *  Created on: 17.07.2008
 *      Author: theide
 */

#ifndef XACMLREQUESTDOCUMENT_H_
#define XACMLREQUESTDOCUMENT_H_

#include <vector>
#include <string>
#include <stdexcept>

#include "XacmlAttribute.h"

namespace xacml {

using namespace std;

typedef enum{
	SUBJECT,
	RESOURCE,
	ENVIRONMENT,
	ACTION
} XacmlSection;

#define SUBJECT_ID_URN "urn:oasis:names:tc:xacml:1.0:subject:subject-id"

class XacmlRequestDocument {
private:
	vector<XacmlAttribute*> subjectAttributes;
	vector<XacmlAttribute*> resourceAttributes;
	vector<XacmlAttribute*> actionAttributes;
	vector<XacmlAttribute*> environmentAttributes;

	string subjectCategory;

public:
	XacmlRequestDocument();
	virtual ~XacmlRequestDocument();

	void addSubjectAttributes(vector<XacmlAttribute*> attr);
	void addResourceAttributes(vector<XacmlAttribute*> attr);
	void addActionAttributes(vector<XacmlAttribute*> attr);
	void addEnvironmentAttributes(vector<XacmlAttribute*> attr);

	void setSubjectCategory(string cat);
	string getSubjectCategory();

	string getPrimarySubjectID();

	vector<XacmlAttribute*>::iterator getAttributeIteratorBegin(XacmlSection sect);
	vector<XacmlAttribute*>::iterator getAttributeIteratorEnd(XacmlSection sect);

	string dump();
};

}

#endif /* XACMLREQUESTDOCUMENT_H_ */
