/********************************************************************************
*                                                                               *
* Copyright (c) 2007, Computer Networks and Internet, University of Tübingen,   *
* Sand 13, 72076 Tübingen, Germany                                              *
*                                                                               *
* All rights reserved.                                                          *
*                                                                               *
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Lesser General Public                    *
* License as published by the Free Software Foundation; either                  *
* version 2.1 of the License, or (at your option) any later version.            *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU              *
* Lesser General Public License for more details.                               *
*                                                                               *
* You should have received a copy of the GNU Lesser General Public              *
* License along with this library; if not, write to the Free Software           *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA       *
*                                                                               *
********************************************************************************/
/*
 * XacmlRequestParser.cpp
 *
 *  Created on: 17.07.2008
 *      Author: theide
 */

#include "XacmlRequestParser.h"

namespace xacml {

XacmlRequestParser::XacmlRequestParser() {
	// TODO Auto-generated constructor stub

}

XacmlRequestParser::~XacmlRequestParser() {
	// TODO Auto-generated destructor stub
}

XacmlRequestDocument XacmlRequestParser::parseDocument(string doc) {
	xmlDocPtr _doc;

	xmlXPathContextPtr xpathCtx;

	// Init stuff
	xmlInitParser();
	_doc = xmlParseMemory(doc.c_str(), doc.length());
	if (_doc == NULL) {
		throw std::invalid_argument(__FILE__ ": unable to parse string"+ doc);
	}

	// Create xpath evaluation context
	xpathCtx = xmlXPathNewContext(_doc);

	if(xpathCtx == NULL) {
	    throw std::runtime_error(__FILE__ ": could not create xmlXpathContext\n");
	}

	xmlNodePtr root;
	root=xmlDocGetRootElement(_doc);
	xpathCtx->node = root;

	// Get all namespaces from root element and put into XPath context
	xmlNsPtr* ns = xmlGetNsList(_doc, root);
	//xpathCtx->nsNr = 0;

	//if (xpathCtx->namespaces != NULL) {
	if (ns != NULL) {
		for ( int i = 0; ns[i] != NULL; i++) {

		   int rv = xmlXPathRegisterNs(xpathCtx, (const char*)ns[i]->prefix == NULL ? (xmlChar*) "default" : ns[i]->prefix, ns[i]->href);
		   //printf("prefix->namespace %s -> %s (FAILED=%d)\n", ns[i]->prefix, ns[i]->href, rv );
		}
	}

	XacmlRequestDocument* returnValue = new XacmlRequestDocument();

	vector<XacmlAttribute*> subjectAttr = this->getAttributesOf(xpathCtx, "Subject");
	vector<XacmlAttribute*> resourcesAttr = this->getAttributesOf(xpathCtx, "Resource");
	vector<XacmlAttribute*> actionAttr = this->getAttributesOf(xpathCtx, "Action");
	vector<XacmlAttribute*> environmentAttr = this->getAttributesOf(xpathCtx, "Environment");

	//printf(__FILE__ " - About to add attributes to return value\n");

	returnValue->addSubjectAttributes(subjectAttr);
	returnValue->addResourceAttributes(resourcesAttr);
	returnValue->addActionAttributes(actionAttr);
	returnValue->addEnvironmentAttributes(environmentAttr);

	//printf(__FILE__ " - Setting subject category\n");

	returnValue->setSubjectCategory(this->getSubjectCategory(xpathCtx));


	//printf("DUMP\n\n%s\n", returnValue->dump().c_str());

	xmlXPathFreeContext(xpathCtx);
	xmlFreeDoc(_doc);

	return *returnValue;
}

vector<XacmlAttribute*> XacmlRequestParser::getAttributesOf(xmlXPathContextPtr ctx, string section) {
	xmlNodeSetPtr nodeSet = this->getNodeFromXpath(ctx, "/default:Request/default:" + section + "/default:Attribute");
	xmlNodePtr walker;

	string* attributeId;
	string* type;

	string* attributeValue;

	vector<XacmlAttribute*> returnValue;

	for (int i = 0; i < xmlXPathNodeSetGetLength(nodeSet); i++) {
		attributeId = new string((const char*) xmlGetProp(nodeSet->nodeTab[i], (xmlChar*) "AttributeId"));
		type = new string((const char*) xmlGetProp(nodeSet->nodeTab[i], (xmlChar*) "DataType"));

		XacmlAttribute* attr = new XacmlAttribute(*attributeId, *type);
		returnValue.push_back(attr);

		//printf(__FILE__ " - Found Attribute with id=%s, type=%s\n", attributeId->c_str(), type->c_str());
		for (walker = nodeSet->nodeTab[i]->children; walker != NULL; walker = walker->next) {
			if (0==xmlStrcmp(walker->name, (xmlChar*) "AttributeValue")) {
				attributeValue = new string((const char*) walker->children->content);
				//printf(__FILE__ " - got node of type %d with name %s, content=%s\n", walker->type, walker->name, attributeValue->c_str());

				attr->addValue(*attributeValue);
			}
		}
	}

	return returnValue;
}

	xmlXPathObjectPtr XacmlRequestParser::getObjPtr(xmlXPathContextPtr ctx, string xPathExpr) {
		xmlXPathObjectPtr xpathObj = NULL;

		// Evaluate xpath expression
		xpathObj = xmlXPathEval ((xmlChar*)(xPathExpr.c_str()), ctx);
			if(xpathObj == NULL) {
			xmlXPathFreeContext(ctx);
			throw std::invalid_argument(__FILE__ " Error: unable to evaluate xpath expression " + xPathExpr );
		}
			//printf("Number of results: %d\n", xmlXPathNodeSetGetLength(xpathObj->nodesetval));
			for (int i=0; i < xmlXPathNodeSetGetLength(xpathObj->nodesetval); i++) {
			//printf("Found Node: %s\n", xpathObj->nodesetval->nodeTab[i]->name);
		}
        return xpathObj;
    }

    xmlNodeSetPtr XacmlRequestParser::getNodeFromXpath(xmlXPathContextPtr ctx, string xPathExpr)
    {
        xmlXPathObjectPtr obj = this->getObjPtr(ctx, xPathExpr);
        return obj->nodesetval;
    }

    string XacmlRequestParser::getSubjectCategory(xmlXPathContextPtr ctx)
    {
    	const char* cat;
    	xmlNodeSetPtr nodeSet = this->getNodeFromXpath(ctx, "/default:Request/default:Subject");
    	cat = (const char*) xmlGetProp(nodeSet->nodeTab[0], (xmlChar*) "SubjectCategory");
    	if (cat != NULL) {
    		return string(cat);
    	} else {
    		return string("");
    	}
    }


}
