/********************************************************************************
*                                                                               *
* Copyright (c) 2010, ThereSIS, Romain FERRARI,   								*
* Palaiseau, France                                              				*
*                                                                               *
* All rights reserved.                                                          *
*                                                                               *
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Lesser General Public                    *
* License as published by the Free Software Foundation; either                  *
* version 2.1 of the License, or (at your option) any later version.            *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU              *
* Lesser General Public License for more details.                               *
*                                                                               *
* You should have received a copy of the GNU Lesser General Public              *
* License along with this library; if not, write to the Free Software           *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA       *
*                                                                               *
********************************************************************************/

#include "pam_xacml.h"
#include "core/pam_handler.h"
#include "syslog.h"

/**
 * This file implements functions to be called by PAM
 * 
 * Architecture:
 * 
 *     PAM			BUILD REQUEST                     ASK PDP           RETURN DECISION
 * authenticate --\
 *                 |   /--buildXacmlRequest --\        |--- staticResponse--|
 * setcred --------|__/                        \_____ /---- simpleRequest----\___parseResponse
 *                 |  \                        /      \---- gsoapRequest-----/
 * acct_mgmt -----/    \--converse -----------/        |--- axis2cRequest---|
 */

PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, const char **argv)
{
	openlog("pam_xacml", 0, LOG_AUTHPRIV);
	syslog(LOG_INFO, "pam_sm_authenticate: XACML Authorization without conversation");
	closelog();
	
	return pam_authorize(pamh, flags, argc, argv);
		
}

PAM_EXTERN int pam_sm_setcred(pam_handle_t *pamh, int flags, int argc, const char **argv)
{
	openlog("pam_xacml", 0, LOG_AUTHPRIV);
	syslog(LOG_INFO, "pam_sm_setcred: XACML Authorization without conversation");
	closelog();
	
	return pam_authorize(pamh, flags, argc, argv);
	
}

PAM_EXTERN int pam_sm_acct_mgmt(pam_handle_t *pamh, int flags, int argc, const char **argv)
{
	int ret = 0;

	openlog("pam_xacml", 0, LOG_AUTHPRIV);

	syslog(LOG_INFO, "pam_sm_acct_mgmt: Performing full XACML authorization conversation");
	ret = pam_authorize(pamh, flags, argc, argv);
	if (ret == PAM_SUCCESS)
	{
		syslog(LOG_INFO, "pam_sm_acct_mgmt successful: %d", ret);
		return PAM_SUCCESS;
	}
	else
		syslog(LOG_INFO, "pam_sm_acct_mgmt failed: %d", ret);

	closelog();

	return ret;
}

