#include "pam_xacml_constants.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <security/pam_appl.h>
#include <security/pam_misc.h>

static int errorcounter = 0;
static char convCalled = 0;
char* user;
char* result = "xacml_test_ok";
int verbose=false;
char str[1024];

#define ATTRIBUTE_ID "<Attribute AttributeId=\""
#define ATTRIBUTE_TYPE "\" DataType=\""
#define ATTRIBUTE_FINISH "\"><AttributeValue> %s </AttributeValue></Attribute>"

#define XACML_TEMPLATE "<?xml version=\"1.0\" encoding=\"UTF-8\"?> \
	<Request xmlns=\"urn:oasis:names:tc:xacml:2.0:context:schema:os\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\"urn:oasis:names:tc:xacml:2.0:context:schema:os http://docs.oasis-open.org/xacml/access_control-xacml-2.0-context-schema-os.xsd\"> \
		<Subject> "\
		ATTRIBUTE_ID "urn:oasis:names:tc:xacml:1.0:subject:subject-id" \
		ATTRIBUTE_TYPE "http://www.w3.org/2001/XMLSchema#string" \
		ATTRIBUTE_FINISH \
		"</Subject>" \
		"<Resource> "\
		ATTRIBUTE_ID "sourceAddress" \
		ATTRIBUTE_TYPE "http://www.w3.org/2001/XMLSchema#string" \
		ATTRIBUTE_FINISH \
		"</Resource>" \
		"<Action>" \
		ATTRIBUTE_ID "request-type" \
		ATTRIBUTE_TYPE "http://www.w3.org/2001/XMLSchema#string" \
		ATTRIBUTE_FINISH \
		"</Action></Request>"




void fail(const char* message) {
	if (verbose) printf("Failed: %s\n", message);
	errorcounter++;
}

int pamAuthorisationConversation(int nmsg, const struct pam_message **pam_msg, struct pam_response **resp, void *appData)
{
  *resp = (struct pam_response*)calloc(nmsg, sizeof(struct pam_response));
  if (verbose) printf("Entered pamAuthorisationConversation");
  if (*resp == NULL) return PAM_BUF_ERR;
  convCalled++;


  int i;
  for (i=0; i<nmsg; i++) {
 	 switch (pam_msg[i]->msg_style) {
   	   printf("TYPE %d\n",pam_msg[i]->msg_style);
	   case PAM_AUTHZ_REQUEST:
		 	 if(NULL!=strstr(pam_msg[i]->msg,"urn:oasis:names:tc:xacml:2.0")){
					//Build XACML Request from template
		 		 char xacmlRequest[10000];
				 snprintf(xacmlRequest, 10000-1, XACML_TEMPLATE,user,"????",result);
				 (*resp)->resp = strdup(xacmlRequest);
				 (*resp)->resp_retcode = PAM_SUCCESS;
				 break;
			 } else {//Unsupported authz Standard
				 free(*resp); *resp = NULL;
				 return PAM_SYSTEM_ERR;
			 }
 		 case PAM_AUTHZ_RESPONSE_INFORMATIONAL:
		 	 //MAY ignore XACML Authorization Decision
			 (*resp)->resp = strdup("OK");
			 (*resp)->resp_retcode = PAM_SUCCESS;
			 break;
		 case PAM_AUTHZ_RESPONSE_MANDATORY:
			 //Did not understand XACML Authorization Decision
			 free(*resp); *resp = NULL;
			 return PAM_CONV_ERR;
		 default:
			 free(*resp); *resp = NULL;
			 return PAM_CONV_ERR;
	 }
  }
  return PAM_SUCCESS;
}

void usage(char* prog_name) {
  printf("Usage : %s -h[elp] [-v[erbose]] [-d[ebug]] -n <number_of_loops> -u <user_id> -r <xacml_test_ok | xacml_test_nok>\n",prog_name);
} /* usage() */


int main(int argc, char** argv)
{
  pam_handle_t *pamh = NULL;
  int returnCode;
  struct pam_conv conv = { &pamAuthorisationConversation, NULL };
  int loops=1;


  int opt;
  while ((opt = getopt(argc, argv, "vhn:u:r:")) != -1) {
    switch (opt) {
      case 'u': user = optarg; break;
      case 'r': result = optarg; break;
      case 'n': loops = atoi(optarg); break;
      case 'v': verbose = true; break;
      default: usage(argv[0]); exit(PAM_SESSION_ERR);
      }
    }

/* Check mandatory parameters */
  if  (!(user && strlen(user))) {
     if (verbose) fail("no user provided");
     exit(PAM_USER);
  }

  int i;
  for (i=0; i<loops; i++) {
	  if (verbose) printf("Loop %d - user %s\n",i, user);
	  returnCode = pam_start(result, user, &conv, &pamh); 
	  if (returnCode != PAM_SUCCESS) {
		  sprintf(str,"while calling pam_start (%d)", returnCode);
		  if (verbose) fail(str);
		  exit(PAM_OPEN_ERR);
	  }
  	  if (verbose) printf("pam_start succeeded!\n");  /* PAM_SUCCESS */

	  returnCode = pam_acct_mgmt(pamh, 0);
	  if (returnCode != PAM_SUCCESS) {
		  if (verbose) printf("pam_acct_mgmt: %d \n", returnCode);
		  fail("pam_acct_mgmt");
		  exit(PAM_SERVICE_ERR);
	  }
  	  if (verbose) { /* PAM_SUCCESS */
	  	  if (convCalled) printf("Authorization Succeeded!\n");  
	  	  else fail("Conversation function has not been called by PAM.");
         }
	 pam_end(pamh, PAM_SUCCESS);
  }
 return(0);
}

